program Steep
  ! Pred ---  Prediction in  2D or 3D.
  !
  !  <in.H Pred [options] > out.H
  !  n1,n2,n3 wall sizes
  !  w1,w2,w3 window sizes
  !  k1,k2,k3 number of windows
  !  a1,a2,a3 PEF size.
  !
  !     default a2=2,a3=1 in-line     2-D PEFs only.
  !               a2=1,a3=2 out-of-line components.
  !  postrect=1 post smoothing the 1-axis
  !%
  use sep
  use decon
  use tent
  use patching
  use triangle

  implicit none
  integer, dimension (3)   :: n, w, k, a, gap, lag
  real, dimension (3)   :: d
  real, dimension (:,:,:), pointer :: data, resi, sign
  real, dimension (:,:), pointer  :: pdata, presi, wind
  real     :: signvar, datavar, vel, tgap
  integer     :: i1, i3, postrect1
  character (len = 80)  	:: process

  call sep_init ()
  call from_history (n (1), n (2))
  call from_history ("n3", n (3), 1)

  call from_par ("w1", w (1), min (n (1), 20))
  call from_par ("w2", w (2), min (n (2), 20))
  call from_par ("w3", w (3), min (n (3), 6 ))

  call from_par ("a1", a (1), 5)
  call from_par ("a2", a (2), 2)
  call from_par ("a3", a (3), 1)

  k = 1.5 * n / (w - a + 1.)

  call from_par ("k1", k (1), k (1))
  call from_par ("k2", k (2), k (2))
  call from_par ("k3", k (3), k (3))

  allocate (data (n (1), n (2), n (3)))
  allocate (resi (n (1), n (2), n (3)))
  allocate (sign (n (1), n (2), n (3)))

  call sep_read (data)
  data = data / maxval (abs (data))

  call from_par ("gap1", gap (1), 0)
  call from_par ("gap2", gap (2), 0)
  call from_par ("gap3", gap (3), 0)

  call from_par ("lag1", lag (1), (a (1) + 1)/2)
  call from_par ("lag2", lag (2), 1)
  call from_par ("lag3", lag (3), 1)

  call from_par ("process", process)

  select case (process (1:8))
  case ("SteepCut")
     call from_history ("d1", d (1))
     call from_history ("d2", d (2))
     call from_history ("d3", d (3), 1.)
     call from_par ("vel", vel, 1.7)
     call from_par ("tgap", tgap, .030)
     call decon2_init (w (:2), a (:2), vel, tgap, d (:2))
  case ("lomoplan")
     call from_par ("postrect1", postrect1, 1)
     call decon2_init (w (:2), a (:2), lag (:2), gap (:2))
  case ("predicte")
     call decon2_init (w (:2), a (:2), lag (:2), gap (:2))
  case default
     call erexit ("unknown process: " // process)
  end select

  call sep_close ()

  allocate (wind (w (1), w (2)))
  call tent (a (:2), lag (:2), wind)

  do i3 = 1, n (3)
     pdata => data (:,:,i3)
     presi => resi (:,:,i3)
     call patching (decon2_img, pdata, presi, k (:2), wind)
  end do
  call decon2_close ()

  select case (process (1:8))
  case ("SteepCut")
     signvar = sqrt (sum (resi * resi))
     datavar = sqrt (sum (data * data))
     where (resi == 0.) resi = data * signvar / datavar
     call sep_write (resi)
  case ("predicte")
     sign = data - resi
     signvar = sqrt (sum (sign * sign))
     datavar = sqrt (sum (data * data))
     where (resi == 0.) sign = data * signvar / datavar
     call sep_write (sign)
  case default
     call triangle ((/postrect1, 1, 1 /), resi)
     call sep_write (resi)
  end select

  deallocate (data, resi, sign)
  call exit (0)
end program Steep


