module tv_decon
  use tvdec
  use cgstep_mod
  use solver_mod

  implicit none

contains
  ! Time Variable (TV) Deconvolution
  !
  subroutine tvdecon (rho, gap, na, yy, rr, rmov, niter, nres)
    integer, intent (in)              :: gap, na, niter, nres
    real, intent (in)                 :: rho   
    real,    dimension (:), pointer   :: yy, rr
    real,    dimension (:,:)          :: rmov

    real,    dimension (size (yy)*na) :: mm
    logical, dimension (size (yy)*na) :: ll
    integer                           :: nx
    real, dimension (size (rr))       :: dat

    nx = size (yy) ; dat = 0. 
    mm = 0. ; mm(1) = 1. ; mm(2:nx) = 1.- rho
    ll (:gap*nx) = .true. ; ll (gap*nx+1:) = .false.
    call tvdec_init (yy, na, gap, rho)
    call cgstep_init ()
    call solver (tvdec_op, cgstep, x0 = mm, x = mm, dat = dat, mask = ll, &
    niter = nres , rmov = rmov)
    call solver (tvdec_op, cgstep, x0 = mm, x = mm, dat = dat, mask = ll, &
    niter = niter, res  = rr)
    call cgstep_close ()
    call tvdec_close ()
  end subroutine tvdecon

end module tv_decon
